@@ -24,6 +24,7 @@
 #include <partition.h>
 #include <linux/sizes.h>
 #include <gpio.h>
+#include <mci.h>
 
 #include <generated/mach-types.h>
 
@@ -105,6 +106,7 @@ static struct esdhc_platform_data tx53_sd1_data = {
 	.cd_gpio = TX53_SD1_CD,
 	.cd_type = ESDHC_CD_GPIO,
 	.wp_type = ESDHC_WP_NONE,
+	.caps    = MMC_CAP_4_BIT_DATA,
 };
 
 struct imx_nand_platform_data nand_info = {
