@@ -52,10 +52,13 @@ static int boot_script(char *path)
 		goto out;
 	}
 
-	data.verbose = verbose;
-	data.dryrun = dryrun;
 	bootm_data_init_defaults(&data);
 
+	if (verbose)
+		data.verbose = verbose;
+	if (dryrun)
+		data.dryrun = dryrun;
+
 	ret = bootm_boot(&data);
 	if (ret)
 		pr_err("Booting %s failed: %s\n", basename(path), strerror(-ret));
