@@ -33,6 +33,16 @@ int globalvar_add(const char *name,
 	return 0;
 }
 
+void globalvar_remove(const char *name)
+{
+	struct param_d *param = get_param_by_name(&global_device, name);
+
+	if (!param)
+		return;
+
+	dev_remove_param(param);
+}
+
 static int nv_save(const char *name, const char *val)
 {
 	int fd, ret;
@@ -42,7 +52,7 @@ static int nv_save(const char *name, const char *val)
 	if (ret)
 		return ret;
 
-	fname = asprintf("/env/nv/%s", name);
+	fname = basprintf("/env/nv/%s", name);
 
 	fd = open(fname, O_CREAT | O_WRONLY | O_TRUNC);
 
@@ -51,7 +61,7 @@ static int nv_save(const char *name, const char *val)
 	if (fd < 0)
 		return fd;
 
-	fprintf(fd, "%s", val);
+	dprintf(fd, "%s", val);
 
 	close(fd);
 
@@ -90,6 +100,9 @@ int nvvar_add(const char *name, const char *value)
 	struct param_d *p, *gp;
 	int ret;
 
+	if (!IS_ENABLED(CONFIG_NVVAR))
+		return -ENOSYS;
+
 	gp = get_param_by_name(&nv_device, name);
 	if (gp) {
 		ret = dev_set_param(&global_device, name, value);
@@ -131,11 +144,14 @@ int nvvar_remove(const char *name)
 	struct param_d *p;
 	char *fname;
 
+	if (!IS_ENABLED(CONFIG_NVVAR))
+		return -ENOSYS;
+
 	p = get_param_by_name(&nv_device, name);
 	if (!p)
 		return -ENOENT;
 
-	fname = asprintf("/env/nv/%s", p->name);
+	fname = basprintf("/env/nv/%s", p->name);
 	unlink(fname);
 	free(fname);
 
@@ -153,6 +169,9 @@ int nvvar_load(void)
 	DIR *dir;
 	struct dirent *d;
 
+	if (!IS_ENABLED(CONFIG_NVVAR))
+		return -ENOSYS;
+
 	dir = opendir("/env/nv");
 	if (!dir)
 		return -ENOENT;
@@ -185,7 +204,7 @@ static void device_param_print(struct device_d *dev)
 		const char *p = dev_get_param(dev, param->name);
 		const char *nv = NULL;
 
-		if (dev != &nv_device)
+		if (IS_ENABLED(CONFIG_NVVAR) && dev != &nv_device)
 			nv = dev_get_param(&nv_device, param->name);
 
 		printf("%s%s: %s\n", nv ? "* " : "  ", param->name, p);
@@ -194,6 +213,9 @@ static void device_param_print(struct device_d *dev)
 
 void nvvar_print(void)
 {
+	if (!IS_ENABLED(CONFIG_NVVAR))
+		return;
+
 	device_param_print(&nv_device);
 }
 
@@ -217,7 +239,8 @@ char *globalvar_get_match(const char *match, const char *separator)
 		if (!strncmp(match, param->name, strlen(match))) {
 			const char *p = dev_get_param(&global_device, param->name);
 			if (val) {
-				char *new = asprintf("%s%s%s", val, separator, p);
+				char *new = basprintf("%s%s%s", val,
+							separator, p);
 				free(val);
 				val = new;
 			} else {
@@ -264,7 +287,9 @@ int globalvar_add_simple(const char *name, const char *value)
 static int globalvar_init(void)
 {
 	register_device(&global_device);
-	register_device(&nv_device);
+
+	if (IS_ENABLED(CONFIG_NVVAR))
+		register_device(&nv_device);
 
 	globalvar_add_simple("version", UTS_RELEASE);
 
