@@ -27,6 +27,7 @@
 #include <net.h>
 #include <malloc.h>
 #include <driver.h>
+#include <string.h>
 #include <linux/err.h>
 
 struct param_d *get_param_by_name(struct device_d *dev, const char *name)
@@ -314,10 +315,13 @@ static int param_int_set(struct device_d *dev, struct param_d *p, const char *va
 	if (!val)
 		return -EINVAL;
 
-	*pi->value = simple_strtol(val, NULL, 0);
-
-	if (pi->flags & PARAM_INT_FLAG_BOOL)
-		*pi->value = !!*pi->value;
+	if (pi->flags & PARAM_INT_FLAG_BOOL) {
+		ret = strtobool(val, pi->value);
+		if (ret)
+			return ret;
+	} else {
+		*pi->value = simple_strtol(val, NULL, 0);
+	}
 
 	if (!pi->set)
 		return 0;
@@ -341,7 +345,7 @@ static const char *param_int_get(struct device_d *dev, struct param_d *p)
 	}
 
 	free(p->value);
-	p->value = asprintf(pi->format, *pi->value);
+	p->value = basprintf(pi->format, *pi->value);
 
 	return p->value;
 }
@@ -443,7 +447,7 @@ static const char *param_enum_get(struct device_d *dev, struct param_d *p)
 	free(p->value);
 
 	if (*pe->value >= pe->num_names)
-		p->value = asprintf("invalid:%d", *pe->value);
+		p->value = basprintf("invalid:%d", *pe->value);
 	else
 		p->value = strdup(pe->names[*pe->value]);
 
@@ -556,7 +560,7 @@ struct param_d *dev_add_param_int_ro(struct device_d *dev, const char *name,
 		return ERR_PTR(ret);
 	}
 
-	piro->param.value = asprintf(format, value);
+	piro->param.value = basprintf(format, value);
 
 	return &piro->param;
 }
@@ -582,7 +586,7 @@ struct param_d *dev_add_param_llint_ro(struct device_d *dev, const char *name,
 		return ERR_PTR(ret);
 	}
 
-	piro->param.value = asprintf(format, value);
+	piro->param.value = basprintf(format, value);
 
 	return &piro->param;
 }
